#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Merge parameters from command line into file, replacing %1, %2 ... */
/* Limit to 35 parameters for now */
/* Input comes from standard input, output goes to standard output */
/* substitution: %1...%9 replaced, %x results in x for x not 1...9 */
/*               %A...%Z for parameters 10...35 */

/*
    Copyright (C) 1991, 1998, 2003  Norm London, London Computing

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	norm_london@londoncomputing.com

	Windows:     Compile with Visual Studio 5 or later or gcc 2.9.5 or later
	should work with just about any c compiler.
	cl cmerge.c
	windows or unix using gcc:
	gcc cmerge.c -o cmerge.exe
*/


int main(int argc,char *argv[]) {

	int c, firstarg = 0, argno;
	char ctl = '%', arg[1024];

	if (argc > 1)
		strcpy(arg,argv[1]);
	else
		strcpy(arg,"-?");

	if (argc < 2 || strcmp(arg,"?")==0 || strcmp(arg,"-?")==0 || strcmp(arg,"-$")==0 && argc < 3) {
		fprintf(stderr,"cmerge Version 1.1, %s\n",__DATE__);
		fprintf(stderr,"\nCopyright 2003, Norm London, London Computing.\nnorm_london@londoncomputing.com\n\n");
		fprintf(stderr,"This program is free software; you can redistribute it and/or modify\n");
		fprintf(stderr,"it under the terms of the GNU General Public License as published by\n");
		fprintf(stderr,"the Free Software Foundation; either version 2 of the License, or\n");
		fprintf(stderr,"(at your option) any later version.\n\n");
		fprintf(stderr,"This program is distributed in the hope that it will be useful,\n");
		fprintf(stderr,"but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
		fprintf(stderr,"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
		fprintf(stderr,"GNU General Public License for more details.\n\n");
		fprintf(stderr,"You should have received a copy of the GNU General Public License\n");
		fprintf(stderr,"along with this program; if not, write to the Free Software\n");
		fprintf(stderr,"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n");

		fprintf(stderr,"Usage is: cmerge {-? | [-$] arg1 [arg2 ... arg35] [<input] [>output]}\n");
		fprintf(stderr,"Substitutes ith command line argument for %ci (1 <= i <=9\n",'%');
		fprintf(stderr,"Use %cA ... %cZ for next 26 command line arguments.\n",'%','%');
		fprintf(stderr,"Use -$ flag to use a $ instead of a % as the control character.\n");
		fprintf(stderr,"Use -? to display this help.\n");
		fprintf(stderr,"Returns 0 if merge successful, else error occurred\n");
		return(4);
	}

	// Process -$ if present
    if (strcmp(arg,"-$") == 0) {
		firstarg++;
		ctl = '$';
	}

	// read a character from input and process until EOF
	    while ((c=getc(stdin)) != EOF) {
			if (c == ctl) {
				c = getc(stdin);
			if (c != EOF)
				c = toupper(c);
			switch(c) {
			case EOF:   /* shouldn't happen, but so what */
				return 2;

			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				argno = c - '0' + firstarg;
				if (argno >= argc) {
					fprintf(stderr,"Argument %c is missing.  CMERGE quitting.\n",c);
					return(2);
				}
				fputs(argv[argno],stdout);
				break;

			case 'A':
			case 'B':
			case 'C':
			case 'D':
			case 'E':
			case 'F':
			case 'G':
			case 'H':
			case 'I':
			case 'J':
			case 'K':
			case 'L':
			case 'M':
			case 'N':
			case 'O':
			case 'P':
			case 'Q':
			case 'R':
			case 'S':
			case 'T':
			case 'U':
			case 'V':
			case 'W':
			case 'X':
			case 'Y':
			case 'Z':
				argno = (c-'A'+10) + firstarg;
				if (argno >= argc) {
					fprintf(stderr,"Argument %c is missing.  CMERGE quitting.\n",c);
					return(2);
				}
				fputs(argv[argno],stdout);
				break;

			default:
				putchar(c);
			}
		}
		else
			putchar(c);
	}
	return 0;
}
